@@ -1,5 +1,5 @@
 module YARD
-  VERSION = "0.7.4"
+  VERSION = "0.8.2.1"
 
   # The root path for YARD source libraries
   ROOT = File.expand_path(File.dirname(__FILE__))
@@ -32,15 +32,6 @@ end
 # Keep track of Ruby version for compatibility code
 RUBY19, RUBY18 = *(RUBY_VERSION >= "1.9.1" ? [true, false] : [false, true])
 
-# Whether or not continuations are (properly) supported
-begin
-  begin; require 'continuation'; rescue LoadError; end
-  cc = callcc {|cc| cc }; cc.call if cc
-  CONTINUATIONS_SUPPORTED = true
-rescue Exception
-  CONTINUATIONS_SUPPORTED = false
-end
-
 # Load Ruby core extension classes
 Dir.glob(File.join(YARD::ROOT, 'yard', 'core_ext', '*.rb')).each do |file|
   require file
