@@ -3,7 +3,7 @@ module YARD
     # Lists all constant and method names in the codebase. Uses {Yardoc} --list.
     class List < Command
       def description; 'Lists all constant and methods. Uses `yard doc --list`' end
-      
+
       # Runs the commandline utility, parsing arguments and displaying a
       # list of objects
       #
@@ -14,7 +14,7 @@ module YARD
           puts "Usage: yard list [yardoc_options]"
           puts "Takes the same arguments as yardoc. See yardoc --help"
         else
-          Yardoc.run('--list', *args)
+          Yardoc.run('-c', '--list', *args)
         end
       end
     end
