@@ -217,11 +217,28 @@ module YARD
         @visibility = :public
         @tags = []
         @docstring = Docstring.new('', self)
+        @docstring_extra = nil
+        @docstring_extra_tags = nil
         @namespace = nil
         self.namespace = namespace
         yield(self) if block_given?
       end
 
+      # Copies all data in this object to another code object, except for
+      # uniquely identifying information (path, namespace, name, scope).
+      #
+      # @param [Base] other the object to copy data to
+      # @return [Base] the other object
+      # @since 0.8.0
+      def copy_to(other)
+        copyable_attributes.each do |ivar|
+          ivar = "@#{ivar}"
+          other.instance_variable_set(ivar, instance_variable_get(ivar))
+        end
+        other.docstring = docstring.to_raw
+        other
+      end
+
       # The name of the object
       # @param [Boolean] prefix whether to show a prefix. Implement
       #   this in a subclass to define how the prefix is showed.
@@ -284,7 +301,7 @@ module YARD
 
       # @return [Integer] the object's hash value (for equality checking)
       def hash; path.hash end
-      
+
       # @return [nil] this object does not turn into an array
       def to_ary; nil end
 
@@ -346,6 +363,7 @@ module YARD
         end
       end
 
+      undef docstring
       def docstring
         return @docstring if !@docstring_extra
         case @docstring
@@ -353,7 +371,9 @@ module YARD
           return @docstring_extra
         when Base
           @docstring = @docstring.docstring + @docstring_extra
+          @docstring.add_tag(*@docstring_extra_tags)
           @docstring_extra = nil
+          @docstring_extra_tags = nil
         end
         @docstring
       end
@@ -368,9 +388,12 @@ module YARD
         if comments =~ /\A\s*\(see (\S+)\s*\)(?:\s|$)/
           path, extra = $1, $'
           @docstring_extra = Docstring.new(extra, self)
+          @docstring_extra_tags = Docstring === comments ? comments.tags : []
+          @docstring_extra.add_tag(*@docstring_extra_tags)
           @docstring = Proxy.new(namespace, path)
         else
           @docstring_extra = nil
+          @docstring_extra_tags = nil
           @docstring = Docstring === comments ? comments : Docstring.new(comments, self)
         end
       end
@@ -438,7 +461,7 @@ module YARD
       # @return [String] the rendered template
       # @see Templates::Engine#render
       def format(options = {})
-        options.merge!(:object => self)
+        options = options.merge(:object => self)
         Templates::Engine.render(options)
       end
 
@@ -487,8 +510,6 @@ module YARD
       # @return whether or not this object is a RootObject
       def root?; false end
 
-      protected
-
       # Override this method with a custom component separator. For instance,
       # {MethodObject} implements sep as '#' or '.' (depending on if the
       # method is instance or class respectively). {#path} depends on this
@@ -498,6 +519,23 @@ module YARD
       #   and the name (default is {NSEP})
       def sep; NSEP end
 
+      protected
+
+      # Override this method if your code object subclass does not allow
+      # copying of certain attributes.
+      #
+      # @return [Array<String>] the list of instance variable names (without
+      #   "@" prefix) that should be copied when {#copy_to} is called
+      # @see #copy_to
+      # @since 0.8.0
+      def copyable_attributes
+        vars = instance_variables.map {|ivar| ivar.to_s[1..-1] }
+        vars -= %w(docstring namespace name path)
+        vars
+      end
+
+      private
+
       # Formats source code by removing leading indentation
       #
       # @param [String] source the source code to format
