@@ -4,9 +4,9 @@ module YARD
   module Parser
     module Ruby
       # Ruby 1.9 parser
-      # @attr_reader encoding_line
-      # @attr_reader shebang_line
-      # @attr_reader enumerator
+      # @!attribute [r] encoding_line
+      # @!attribute [r] shebang_line
+      # @!attribute [r] enumerator
       class RubyParser < Parser::Base
         def initialize(source, filename)
           @parser = RipperParser.new(source, filename)
@@ -33,6 +33,7 @@ module YARD
           @source = source
           @tokens = []
           @comments = {}
+          @comments_range = {}
           @comments_flags = {}
           @heredoc_tokens = nil
           @heredoc_state = nil
@@ -40,9 +41,9 @@ module YARD
           @ns_charno = 0
           @list = []
           @charno = 0
-          @groups = []
           @shebang_line = nil
           @encoding_line = nil
+          @file_encoding = nil
         end
 
         def parse
@@ -58,6 +59,15 @@ module YARD
           ast.children
         end
 
+        def file_encoding
+          return nil unless defined?(::Encoding)
+          return @file_encoding if @file_encoding
+          return Encoding.default_internal unless @encoding_line
+          if match = @encoding_line.match(SourceParser::ENCODING_LINE)
+            @file_encoding = match.captures.last
+          end
+        end
+
         private
 
         MAPPINGS = {
@@ -234,6 +244,11 @@ module YARD
             @tokens[-1] = [:symbol, ":" + data]
           elsif @heredoc_state == :started
             @heredoc_tokens << [token, data]
+
+            # fix ripper encoding of heredoc bug
+            # (see http://bugs.ruby-lang.org/issues/6200)
+            data.force_encoding(file_encoding) if file_encoding
+
             @heredoc_state = :ended if token == :heredoc_end
           elsif (token == :nl || token == :comment) && @heredoc_state == :ended
             @heredoc_tokens.unshift([token, data])
@@ -244,7 +259,7 @@ module YARD
             @tokens << [token, data]
             if token == :heredoc_beg
               @heredoc_state = :started
-              @heredoc_tokens = [] 
+              @heredoc_tokens = []
             end
           end
         end
@@ -256,6 +271,7 @@ module YARD
         undef on_bare_assoc_hash
         undef on_assoclist_from_args
         undef on_aref
+        undef on_aref_field
         undef on_lbracket
         undef on_rbracket
         undef on_qwords_new
@@ -273,6 +289,9 @@ module YARD
         undef on_embdoc
         undef on_embdoc_end
         undef on_parse_error
+        undef on_bodystmt
+        undef on_top_const_ref
+        undef on_const_path_ref
 
         def on_program(*args)
           args.first
@@ -298,7 +317,7 @@ module YARD
         def on_assoclist_from_args(*args)
           args.first
         end
-        
+
         def on_unary(op, val)
           map = @map[op.to_s[0,1]]
           lstart, sstart = *(map ? map.pop : [lineno, @ns_charno - 1])
@@ -309,12 +328,19 @@ module YARD
         end
 
         def on_aref(*args)
+          @map[:lbracket].pop
           ll, lc = *@map[:aref].pop
           sr = args.first.source_range.first..lc
           lr = args.first.line_range.first..ll
           AstNode.new(:aref, args, :char => sr, :line => lr)
         end
-        
+
+        def on_aref_field(*args)
+          @map[:lbracket].pop
+          AstNode.new(:aref_field, args,
+                      :listline => lineno..lineno, :listchar => charno...charno)
+        end
+
         def on_array(other)
           node = AstNode.node_class_for(:array).new(:array, [other])
           map = @map[MAPPINGS[node.type]]
@@ -323,12 +349,16 @@ module YARD
             node.source_range = Range.new(sstart, @ns_charno - 1)
             node.line_range = Range.new(lstart, lineno)
           else
+            sstart = other.source_range.begin
+            lstart = other.line_range.begin
+            node.source_range = Range.new(sstart, @ns_charno - 1)
+            node.line_range = Range.new(lstart, lineno)
             node.source_range = other.source_range
             node.line_range = other.line_range
           end
           node
         end
-        
+
         def on_lbracket(tok)
           (@map[:lbracket] ||= []) << [lineno, charno]
           visit_ns_token(:lbracket, tok, false)
@@ -366,7 +396,7 @@ module YARD
             end
           eof
         end
-        
+
         def on_qwords_new(*args)
           node = LiteralNode.new(:qwords_literal, args)
           if @map[:qwords_beg]
@@ -376,7 +406,7 @@ module YARD
           end
           node
         end
-        
+
         def on_qwords_add(list, item)
           last = @source[@ns_charno,1] == "\n" ? @ns_charno - 1 : @ns_charno
           list.source_range = (list.source_range.first..last)
@@ -455,32 +485,30 @@ module YARD
             end
           end
 
+          ch = charno
           visit_ns_token(:comment, comment)
           if not_comment
             @last_ns_token = nil
             return
           end
-          case comment
-          when /\A#+ @group\s+(.+)\s*\Z/
-            @groups.unshift [lineno, $1]
-            return
-          when /\A#+ @endgroup\s*\Z/
-            @groups.unshift [lineno, nil]
-            return
-          end
 
+          source_range = ch..(charno-1)
           comment = comment.gsub(/^(\#+)\s{0,1}/, '').chomp
           append_comment = @comments[lineno - 1]
+
           hash_flag = $1 == '##' ? true : false
 
           if append_comment && @comments_last_column == column
             @comments.delete(lineno - 1)
             @comments_flags[lineno] = @comments_flags[lineno - 1]
             @comments_flags.delete(lineno - 1)
+            range = @comments_range.delete(lineno - 1)
+            source_range = range.first..source_range.last
             comment = append_comment + "\n" + comment
           end
 
           @comments[lineno] = comment
+          @comments_range[lineno] = source_range
           @comments_flags[lineno] = hash_flag if !append_comment
           @comments_last_column = column
         end
@@ -511,22 +539,67 @@ module YARD
             (node.line - 2).upto(node.line) do |line|
               comment = @comments[line]
               if comment && !comment.empty?
-                node.docstring_hash_flag = @comments_flags[line]
-                node.docstring = comment
-                node.docstring_range = ((line - comment.count("\n"))..line)
-                @comments.delete(line)
-                @comments_flags.delete(line)
+                add_comment(line, node)
                 break
               end
             end
-            if node.type == :def || node.type == :defs || node.call?
-              @groups.each do |group|
-                if group.first < node.line
-                  break node.group = group.last
-                end
+
+            @comments.keys.each do |line|
+              if node.line > line
+                add_comment(line, nil, node)
               end
             end
           end
+
+          # insert any lone unadded comments before node
+          root.traverse do |node|
+            next if node.type == :list || node.parent.type != :list
+            @comments.keys.each do |line|
+              if node.line_range.include?(line)
+                pick = nil
+                node.traverse do |subnode|
+                  next unless subnode.type == :list
+                  pick ||= subnode
+                  next unless subnode.line_range.include?(line)
+                  pick = subnode
+                end
+                add_comment(line, nil, pick, true) if pick
+              end
+            end
+          end if @comments.size > 0
+
+          # insert all remaining comments
+          @comments.each do |line, comment|
+            add_comment(line, nil, root, true)
+          end
+
+          @comments = {}
+        end
+
+        def add_comment(line, node = nil, before_node = nil, into = false)
+          comment = @comments[line]
+          source_range = @comments_range[line]
+          line_range = ((line - comment.count("\n"))..line)
+          if node.nil?
+            node = CommentNode.new(:comment, [comment], :line => line_range, :char => source_range)
+            if into
+              before_node.push(node)
+              before_node.unfreeze
+              node.parent = before_node
+            elsif before_node
+              parent_node = before_node.parent
+              idx = parent_node.index(before_node)
+              parent_node.insert(idx, node)
+              parent_node.unfreeze
+              node.parent = parent_node
+            end
+          end
+          node.docstring = comment
+          node.docstring_hash_flag = @comments_flags[line]
+          node.docstring_range = line_range
+          @comments.delete(line)
+          @comments_range.delete(line)
+          @comments_flags.delete(line)
         end
 
         def freeze_tree(node = nil)
