@@ -1,8 +1,6 @@
 require 'stringio'
 require 'ostruct'
 
-begin require 'continuation'; rescue LoadError; end
-
 module YARD
   module Parser
     # Raised when an object is recognized but cannot be documented. This
@@ -13,19 +11,39 @@ module YARD
     # Raised when the parser sees a Ruby syntax error
     class ParserSyntaxError < UndocumentableError; end
 
-    # A LoadOrderError occurs when a handler needs to modify a
-    # {CodeObjects::NamespaceObject} (usually by adding a child to it)
-    # that has not yet been resolved.
+    # Responsible for parsing a list of files in order. The
+    # {#parse} method of this class can be called from the
+    # {SourceParser#globals} globals state list to re-enter
+    # parsing for the remainder of files in the list recursively.
     #
-    # @see Handers::Base#ensure_loaded!
-    class LoadOrderError < Exception
-      # @return [Continuation] the context representing the
-      #   point at which the load order error occurred.
-      attr_accessor :context
-      
-      # @param [Continuation] context see {#context}
-      def initialize(context)
-        @context = context
+    # @see Processor#parse_remaining_files
+    class OrderedParser
+      # @return [Array<String>] the list of remaining files to parse
+      attr_accessor :files
+
+      # Creates a new OrderedParser with the global state and a list
+      # of files to parse.
+      #
+      # @note OrderedParser sets itself as the +ordered_parser+ key on
+      #   global_state for later use in {Handlers::Processor}.
+      # @param [OpenStruct] global_state a structure containing all global
+      #   state during parsing
+      # @param [Array<String>] files the list of files to parse
+      def initialize(global_state, files)
+        @global_state = global_state
+        @files = files.dup
+        @global_state.ordered_parser = self
+      end
+
+      # Parses the remainder of the {#files} list.
+      #
+      # @see Processor#parse_remaining_files
+      def parse
+        while file = files.shift
+          log.capture("Parsing #{file}") do
+            SourceParser.new(SourceParser.parser_type, @global_state).parse(file)
+          end
+        end
       end
     end
 
@@ -44,7 +62,7 @@ module YARD
     class SourceParser
       SHEBANG_LINE  = /\A\s*#!\S+/
       ENCODING_LINE = /\A(?:\s*#*!.*\r?\n)?\s*(?:#+|\/\*+|\/\/+).*coding\s*[:=]{1,2}\s*([a-z\d_\-]+)/i
-      
+
       # Byte order marks for various encodings
       # @since 0.7.0
       ENCODING_BYTE_ORDER_MARKS = {
@@ -163,20 +181,20 @@ module YARD
         def validated_parser_type(type)
           !defined?(::Ripper) && type == :ruby ? :ruby18 : type
         end
-        
+
         # @group Parser Callbacks
-        
-        # Registers a callback to be called before a list of files is parsed 
-        # via {parse}. The block passed to this method will be called on 
+
+        # Registers a callback to be called before a list of files is parsed
+        # via {parse}. The block passed to this method will be called on
         # subsequent parse calls.
-        # 
+        #
         # @example Installing a simple callback
         #   SourceParser.before_parse_list do |files, globals|
         #     puts "Starting to parse..."
         #   end
         #   YARD.parse('lib/**/*.rb')
         #   # prints "Starting to parse..."
-        # 
+        #
         # @example Setting global state
         #   SourceParser.before_parse_list do |files, globals|
         #     globals.method_count = 0
@@ -190,16 +208,16 @@ module YARD
         #   end
         #   YARD.parse
         #   # Prints: "Found 37 methods"
-        # 
+        #
         # @example Using a global callback to cancel parsing
         #   SourceParser.before_parse_list do |files, globals|
         #     return false if files.include?('foo.rb')
         #   end
-        # 
+        #
         #   YARD.parse(['foo.rb', 'bar.rb']) # callback cancels this method
         #   YARD.parse('bar.rb') # parses normally
-        # 
-        # @yield [files, globals] the yielded block is called once before 
+        #
+        # @yield [files, globals] the yielded block is called once before
         #   parsing all files
         # @yieldparam [Array<String>] files the list of files that will be parsed.
         # @yieldparam [OpenStruct] globals a global structure to store arbitrary
@@ -213,18 +231,18 @@ module YARD
         def before_parse_list(&block)
           before_parse_list_callbacks << block
         end
-        
-        # Registers a callback to be called after a list of files is parsed 
-        # via {parse}. The block passed to this method will be called on 
+
+        # Registers a callback to be called after a list of files is parsed
+        # via {parse}. The block passed to this method will be called on
         # subsequent parse calls.
-        # 
+        #
         # @example Printing results after parsing occurs
         #   SourceParser.after_parse_list do
         #     puts "Finished parsing!"
         #   end
         #   YARD.parse
         #   # Prints "Finished parsing!" after parsing files
-        # @yield [files, globals] the yielded block is called once before 
+        # @yield [files, globals] the yielded block is called once before
         #   parsing all files
         # @yieldparam [Array<String>] files the list of files that will be parsed.
         # @yieldparam [OpenStruct] globals a global structure to store arbitrary
@@ -237,14 +255,14 @@ module YARD
         def after_parse_list(&block)
           after_parse_list_callbacks << block
         end
-        
-        # Registers a callback to be called before an individual file is parsed. 
-        # The block passed to this method will be called on subsequent parse 
+
+        # Registers a callback to be called before an individual file is parsed.
+        # The block passed to this method will be called on subsequent parse
         # calls.
-        # 
+        #
         # To register a callback that is called before the entire list of files
         # is processed, see {before_parse_list}.
-        # 
+        #
         # @example Installing a simple callback
         #   SourceParser.before_parse_file do |parser|
         #     puts "I'm parsing #{parser.file}"
@@ -254,18 +272,18 @@ module YARD
         #   "I'm parsing lib/foo.rb"
         #   "I'm parsing lib/foo_bar.rb"
         #   "I'm parsing lib/last_file.rb"
-        # 
+        #
         # @example Cancel parsing of any test_*.rb files
         #   SourceParser.before_parse_file do |parser|
         #     return false if parser.file =~ /^test_.+\.rb$/
         #   end
-        # 
+        #
         # @yield [parser] the yielded block is called once before each
         #   file that is parsed. This might happen many times for a single
         #   codebase.
         # @yieldparam [SourceParser] parser the parser object that will {#parse}
         #   the file.
-        # @yieldreturn [Boolean] if the block returns +false+, parsing for 
+        # @yieldreturn [Boolean] if the block returns +false+, parsing for
         #   the file is cancelled.
         # @return [Proc] the yielded block
         # @see after_parse_file
@@ -274,14 +292,14 @@ module YARD
         def before_parse_file(&block)
           before_parse_file_callbacks << block
         end
-        
-        # Registers a callback to be called after an individual file is parsed. 
-        # The block passed to this method will be called on subsequent parse 
+
+        # Registers a callback to be called after an individual file is parsed.
+        # The block passed to this method will be called on subsequent parse
         # calls.
-        # 
+        #
         # To register a callback that is called after the entire list of files
         # is processed, see {after_parse_list}.
-        # 
+        #
         # @example Printing the length of each file after it is parsed
         #   SourceParser.after_parse_file do |parser|
         #     puts "#{parser.file} is #{parser.contents.size} characters"
@@ -290,8 +308,8 @@ module YARD
         #   # prints:
         #   "lib/foo.rb is 1240 characters"
         #   "lib/foo_bar.rb is 248 characters"
-        # 
-        # @yield [parser] the yielded block is called once after each file 
+        #
+        # @yield [parser] the yielded block is called once after each file
         #   that is parsed. This might happen many times for a single codebase.
         # @yieldparam [SourceParser] parser the parser object that parsed
         #   the file.
@@ -303,104 +321,88 @@ module YARD
         def after_parse_file(&block)
           after_parse_file_callbacks << block
         end
-        
+
         # @return [Array<Proc>] the list of callbacks to be called before
         #   parsing a list of files. Should only be used for testing.
         # @since 0.7.0
         def before_parse_list_callbacks
           @before_parse_list_callbacks ||= []
         end
-        
+
         # @return [Array<Proc>] the list of callbacks to be called after
         #   parsing a list of files. Should only be used for testing.
         # @since 0.7.0
         def after_parse_list_callbacks
           @after_parse_list_callbacks ||= []
         end
-        
+
         # @return [Array<Proc>] the list of callbacks to be called before
         #   parsing a file. Should only be used for testing.
         # @since 0.7.0
         def before_parse_file_callbacks
           @before_parse_file_callbacks ||= []
         end
-        
+
         # @return [Array<Proc>] the list of callbacks to be called after
         #   parsing a file. Should only be used for testing.
         # @since 0.7.0
         def after_parse_file_callbacks
           @after_parse_file_callbacks ||= []
         end
-        
+
         # @endgroup
 
         private
 
-        # Parses a list of files in a queue. If a {LoadOrderError} is caught,
-        # the file is moved to the back of the queue with a Continuation object
-        # that can continue processing the file.
+        # Parses a list of files in a queue.
         #
         # @param [Array<String>] files a list of files to queue for parsing
         # @return [void]
         def parse_in_order(*files)
           global_state = OpenStruct.new
           files = files.sort_by {|x| x.length if x }
-          files_copy = files.dup
-          
+
           before_parse_list_callbacks.each do |cb|
-            return if cb.call(files_copy, global_state) == false
+            return if cb.call(files, global_state) == false
           end
-          
-          while file = files.shift
-            begin
-              if file.is_a?(Array) && file.last.is_a?(Continuation)
-                log.debug("Re-processing #{file.first}")
-                file.last.call
-              elsif file.is_a?(String)
-                log.debug("Processing #{file}...")
-                new(parser_type, true, global_state).parse(file)
-              end
-            rescue LoadOrderError => e
-              # Out of order file. Push the context to the end and we'll call it
-              files.push([file, e.context])
-            end
-          end
-          
+
+          OrderedParser.new(global_state, files).parse
+
           after_parse_list_callbacks.each do |cb|
-            cb.call(files_copy, global_state)
+            cb.call(files, global_state)
           end
         end
       end
 
       register_parser_type :ruby,   Ruby::RubyParser
       register_parser_type :ruby18, Ruby::Legacy::RubyParser
-      register_parser_type :c,      CParser, ['c', 'cc', 'cxx', 'cpp']
+      register_parser_type :c,      C::CParser, ['c', 'cc', 'cxx', 'cpp']
 
       self.parser_type = :ruby
 
       # @return [String] the filename being parsed by the parser.
-      attr_reader :file
+      attr_accessor :file
 
       # @return [Symbol] the parser type associated with the parser instance.
       #   This should be set by the {#initialize constructor}.
       attr_reader :parser_type
-      
+
       # @return [OpenStruct] an open struct containing arbitrary global state
       #   shared between files and handlers.
       # @since 0.7.0
       attr_reader :globals
-      
+
       # @return [String] the contents of the file to be parsed
       # @since 0.7.0
       attr_reader :contents
 
-      # Creates a new parser object for code parsing with a specific parser type.
+      # @overload initialize(parser_type = SourceParser.parser_type, globals = nil)
+      #   Creates a new parser object for code parsing with a specific parser type.
       #
-      # @param [Symbol] parser_type the parser type to use
-      # @param [Boolean] load_order_errors whether or not to raise the {LoadOrderError}
-      # @param [OpenStruct] globals global state to be re-used across separate source files
-      def initialize(parser_type = SourceParser.parser_type, load_order_errors = false, globals = nil)
-        @load_order_errors = load_order_errors
+      #   @param [Symbol] parser_type the parser type to use
+      #   @param [OpenStruct] globals global state to be re-used across separate source files
+      def initialize(parser_type = SourceParser.parser_type, globals1 = nil, globals2 = nil)
+        globals = [true, false].include?(globals1) ? globals2 : globals1
         @file = '(stdin)'
         @globals = globals || OpenStruct.new
         self.parser_type = parser_type
@@ -430,25 +432,25 @@ module YARD
 
         @contents = content
         @parser = parser_class.new(content, file)
-        
+
         self.class.before_parse_file_callbacks.each do |cb|
           return @parser if cb.call(self) == false
         end
-          
+
         @parser.parse
         post_process
-        
+
         self.class.after_parse_file_callbacks.each do |cb|
           cb.call(self)
         end
-        
+
         @parser
       rescue ArgumentError, NotImplementedError => e
         log.warn("Cannot parse `#{file}': #{e.message}")
-        log.backtrace(e) if log.level >= Logger::DEBUG
+        log.backtrace(e, :warn)
       rescue ParserSyntaxError => e
         log.warn(e.message.capitalize)
-        log.backtrace(e) if log.level >= Logger::DEBUG
+        log.backtrace(e, :warn)
       end
 
       # Tokenizes but does not parse the block of code using the current {#parser_type}
@@ -488,7 +490,7 @@ module YARD
       def post_process
         return unless @parser.respond_to? :enumerator
         return unless enumerator = @parser.enumerator
-        post = Handlers::Processor.new(@file, @load_order_errors, @parser_type, @globals)
+        post = Handlers::Processor.new(self)
         post.process(enumerator)
       end
 
