@@ -33,7 +33,7 @@ module YARD
       # @param [Symbol] markup examples are +:markdown+, +:textile+, +:rdoc+.
       #   To add a custom markup type, see {MarkupHelper}
       # @return [String] the HTML
-      def htmlify(text, markup = options[:markup])
+      def htmlify(text, markup = options.markup)
         markup_meth = "html_markup_#{markup}"
         return text unless respond_to?(markup_meth)
         return "" unless text
@@ -44,14 +44,9 @@ module YARD
           html = html.encode(:invalid => :replace, :replace => '?')
         end
         html = resolve_links(html)
-        html = html.gsub(/<pre\s*(?:lang="(.+?)")?>(?:\s*<code>)?(.+?)(?:<\/code>\s*)?<\/pre>/m) do
-          language = $1
-          string   = $2 
-          
-          string = html_syntax_highlight(CGI.unescapeHTML(string), language) unless options[:no_highlight]
-          classes = ['code', language].compact.join(' ')
-          %Q{<pre class="#{classes}">#{string}</pre>}
-        end unless [:text, :none, :pre].include?(markup)
+        unless [:text, :none, :pre].include?(markup)
+          html = parse_codeblocks(html)
+        end
         html
       end
 
@@ -128,11 +123,11 @@ module YARD
       def html_markup_html(text)
         text
       end
-      
+
       # Highlights Ruby source. Similar to {#html_syntax_highlight}, but
       # this method is meant to be called from {#htmlify} when markup is
       # set to "ruby".
-      # 
+      #
       # @param [String] source the Ruby source
       # @return [String] the highlighted HTML
       # @since 0.7.0
@@ -153,21 +148,15 @@ module YARD
       #   +html_syntax_highlight_TYPE+ in this class.
       #
       # @param [String] source the source code to highlight
-      # @param [Symbol] type the language type (:ruby, :plain, etc). Use
+      # @param [Symbol, String] type the language type (:ruby, :plain, etc). Use
       #   :plain for no syntax highlighting.
       # @return [String] the highlighted source
       def html_syntax_highlight(source, type = nil)
         return "" unless source
-        return h(source) if options[:no_highlight]
-
-        type ||= object.source_type || :ruby
-
-        # handle !!!LANG prefix to send to html_syntax_highlight_LANG
-        if source =~ /\A(?:[ \t]*\r?\n)?[ \t]*!!!([\w.+-]+)[ \t]*\r?\n/
-          type, source = $1, $'
-          source = $'
-        end
+        return h(source) unless options.highlight
 
+        new_type, source = parse_lang_for_codeblock(source)
+        type ||= new_type || :ruby
         meth = "html_syntax_highlight_#{type}"
         respond_to?(meth) ? send(meth, source) : h(source)
       end
@@ -201,12 +190,14 @@ module YARD
           next(match[1..-1]) if escape
 
           next(match) if name[0,1] == '|'
-          
+
           if name == '<a' && title =~ /href=["'](.+?)["'].*>.*<\/a>\s*(.*)\Z/
             name, title = $1, $2
             title = nil if title.empty?
           end
-          
+
+          name = CGI.unescapeHTML(name)
+
           if object.is_a?(String)
             object
           else
@@ -235,14 +226,14 @@ module YARD
         return title || file.title unless serializer
         link_url(url_for_file(file, anchor), title || file.title)
       end
-      
+
       # (see BaseHelper#link_include_file)
       def link_include_file(file)
         unless file.is_a?(CodeObjects::ExtraFileObject)
           file = CodeObjects::ExtraFileObject.new(file)
         end
         file.attributes[:markup] ||= markup_for_file('', file.filename)
-        htmlify(file.contents, file.attributes[:markup] || options[:markup])
+        htmlify(file.contents, file.attributes[:markup] || options.markup)
       end
 
       # (see BaseHelper#link_include_object)
@@ -259,7 +250,14 @@ module YARD
         elsif otitle
           title = otitle.to_s
         elsif object.is_a?(CodeObjects::Base)
-          title = h(object.relative_path(obj))
+          # Check if we're linking to a class method in the current
+          # object. If we are, create a title in the format of
+          # "CurrentClass.method_name"
+          if obj.is_a?(CodeObjects::MethodObject) && obj.scope == :class && obj.parent == object
+            title = h([object.name, obj.sep, obj.name].join)
+          else
+            title = h(object.relative_path(obj))
+          end
         else
           title = h(obj.to_s)
         end
@@ -324,7 +322,7 @@ module YARD
           fromobj = object
           if object.is_a?(CodeObjects::Base) &&
               !object.is_a?(CodeObjects::NamespaceObject)
-            fromobj = fromobj.namespace
+            fromobj = owner
           end
 
           from = serializer.serialized_path(fromobj)
@@ -348,7 +346,7 @@ module YARD
           fromobj = fromobj.namespace
         end
         from = serializer.serialized_path(fromobj)
-        if filename == options[:readme]
+        if filename == options.readme
           path = 'index.html'
         else
           path = serializer.serialized_path(filename)
@@ -358,6 +356,39 @@ module YARD
         link
       end
 
+      # Returns the URL for a list type
+      #
+      # @param [String, Symbol] type the list type to generate a URL for
+      # @return [String] the URL pointing to the list
+      # @since 0.8.0
+      def url_for_list(type)
+        url_for_file("#{type}_list.html")
+      end
+
+      # Returns the URL for the frameset page
+      #
+      # @return [String] the URL pointing to the frames page
+      # @since 0.8.0
+      def url_for_frameset
+        url_for_file("frames.html")
+      end
+
+      # Returns the URL for the main page (README or alphabetic index)
+      #
+      # @return [String] the URL pointing to the first main page the
+      #   user should see.
+      def url_for_main
+        url_for_file("index.html")
+      end
+
+      # Returns the URL for the alphabetic index page
+      #
+      # @return [String] the URL pointing to the first main page the
+      #   user should see.
+      def url_for_index
+        url_for_file("_index.html")
+      end
+
       # @group Formatting Objects and Attributes
 
       # Formats a list of objects and links them
@@ -402,7 +433,7 @@ module YARD
           meth = meth.object
         end
 
-        type = options[:default_return] || ""
+        type = options.default_return || ""
         if meth.tag(:return) && meth.tag(:return).types
           types = meth.tags(:return).map {|t| t.types ? t.types : [] }.flatten.uniq
           first = link ? h(types.first) : format_types([types.first], false)
@@ -412,7 +443,7 @@ module YARD
             type = first + '<sup>+</sup>'
           elsif types.size > 2
             type = [first, '...'].join(', ')
-          elsif types == ['void'] && options[:hide_void_return]
+          elsif types == ['void'] && options.hide_void_return
             type = ""
           else
             type = link ? h(types.join(", ")) : format_types(types, false)
@@ -460,7 +491,8 @@ module YARD
             link_title = "#{h name} (#{meth.type})"
           end
           obj = meth.respond_to?(:object) ? meth.object : meth
-          link_url(url_for(obj), title, :title => link_title) + extras_text
+          url = url_for(object, obj)
+          link_url(url, title, :title => link_title) + extras_text
         else
           title + extras_text
         end
@@ -514,6 +546,43 @@ module YARD
         end
         meth
       end
+
+      # Parses !!!lang out of codeblock, returning the codeblock language
+      # followed by the source code.
+      #
+      # @param [String] source the source code whose language to determine
+      # @return [Array(String, String)] the language, if any, and the
+      #   remaining source
+      # @since 0.7.5
+      def parse_lang_for_codeblock(source)
+        type = nil
+        if source =~ /\A(?:[ \t]*\r?\n)?[ \t]*!!!([\w.+-]+)[ \t]*\r?\n/
+          type, source = $1, $'
+        end
+
+        [type, source]
+      end
+
+      # Parses code blocks out of html and performs syntax highlighting
+      # on code inside of the blocks.
+      #
+      # @param [String] html the html to search for code in
+      # @return [String] highlighted html
+      # @see #html_syntax_highlight
+      def parse_codeblocks(html)
+        html.gsub(/<pre\s*(?:lang="(.+?)")?>(?:\s*<code\s*(?:class="(.+?)")?\s*>)?(.+?)(?:<\/code>\s*)?<\/pre>/m) do
+          string = $3
+          # handle !!!LANG prefix to send to html_syntax_highlight_LANG
+          language, _ = parse_lang_for_codeblock(string)
+          language ||= $1 || $2 || object.source_type
+
+          if options.highlight
+            string = html_syntax_highlight(CGI.unescapeHTML(string), language)
+          end
+          classes = ['code', language].compact.join(' ')
+          %Q{<pre class="#{classes}"><code>#{string}</code></pre>}
+        end
+      end
     end
   end
 end
